#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = "Christian Heider Nielsen"
__doc__ = r"""

           Created on 11-12-2020
           """

from pathlib import Path

from warg.imports import import_warning

import_warning(Path(__file__).with_suffix("").name)

A_CONSTANT = 2
ANOTHER_CONSTANT = 42
